@@ -27,7 +27,10 @@
  *
  */
 
-#ifndef TINS_DOT11_DOT11_CONTROL_H
+#include "../config.h"
+
+#if !defined(TINS_DOT11_DOT11_CONTROL_H) && defined(HAVE_DOT11)
+
 #define TINS_DOT11_DOT11_CONTROL_H
 
 #include "../dot11/dot11_base.h"
