@@ -27,7 +27,10 @@
  *
  */
 
-#ifndef TINS_DOT11_DOT11_MGMT_H
+#include "../config.h"
+
+#if !defined(TINS_DOT11_DOT11_MGMT_H) && defined(HAVE_DOT11)
+
 #define TINS_DOT11_DOT11_MGMT_H
 
 #include <vector>
@@ -363,7 +366,9 @@ public:
 
     } TINS_END_PACK;
     
-    TINS_BEGIN_PACK
+    /**
+     * The type used to store the FS parameters set option data.
+     */
     struct fh_params_set {
         uint16_t dwell_time;
         uint8_t hop_set, hop_pattern, hop_index;
@@ -374,9 +379,13 @@ public:
           uint8_t hop_pattern, uint8_t hop_index) 
         : dwell_time(dwell_time), hop_set(hop_set), 
           hop_pattern(hop_pattern), hop_index(hop_index) {}
-    } TINS_END_PACK;
+
+        static fh_params_set from_option(const option &opt);
+    };
     
-    TINS_BEGIN_PACK
+    /**
+     * The type used to store the CF parameters set option data.
+     */
     struct cf_params_set {
         uint8_t cfp_count, cfp_period;
         uint16_t cfp_max_duration, cfp_dur_remaining;
@@ -388,8 +397,13 @@ public:
         : cfp_count(cfp_count), cfp_period(cfp_period), 
           cfp_max_duration(cfp_max_duration), 
           cfp_dur_remaining(cfp_dur_remaining) {}
-    } TINS_END_PACK;
+
+        static cf_params_set from_option(const option &opt);
+    };
     
+    /**
+     * The type used to store the IBSS DFS parameters option data.
+     */
     struct ibss_dfs_params {
         static const size_t minimum_size = address_type::address_size + sizeof(uint8_t) + 2 * sizeof(uint8_t);
         
@@ -403,39 +417,52 @@ public:
           uint8_t recovery_interval, const channels_type &channels)
         : dfs_owner(addr), recovery_interval(recovery_interval),
           channel_map(channels) {}
+
+        static ibss_dfs_params from_option(const option &opt);
     };
     
+    /**
+     * The type used to store the Country parameters option data.
+     */
     struct country_params {
-        typedef std::vector<uint8_t> container_type;
         // String identifier: 3 bytes
         static const size_t minimum_size = 3 + sizeof(uint8_t) * 3;
         
         std::string country;
-        container_type first_channel, number_channels, max_transmit_power;
+        byte_array first_channel, number_channels, max_transmit_power;
         
         country_params() {}
         
-        country_params(const std::string &country, const container_type &first,
-          const container_type &number, const container_type &max) 
+        country_params(const std::string &country, const byte_array &first,
+          const byte_array &number, const byte_array &max) 
         : country(country), first_channel(first), number_channels(number),
           max_transmit_power(max) {}
+
+        static country_params from_option(const option &opt);
     };
     
+    /**
+     * The type used to store the FH pattern option data.
+     */
     struct fh_pattern_type {
-        typedef std::vector<uint8_t> container_type;
         static const size_t minimum_size = sizeof(uint8_t) * 4;
         
         uint8_t flag, number_of_sets, modulus, offset;
-        container_type random_table;
+        byte_array random_table;
         
         fh_pattern_type() {}
         
         fh_pattern_type(uint8_t flag, uint8_t sets, uint8_t modulus,
-          uint8_t offset, const container_type& table) 
+          uint8_t offset, const byte_array& table) 
         : flag(flag), number_of_sets(sets), modulus(modulus), 
           offset(offset), random_table(table) {}
+
+        static fh_pattern_type from_option(const option &opt);
     };
     
+    /**
+     * The type used to store the Channel Switch option data.
+     */
     struct channel_switch_type {
         uint8_t switch_mode, new_channel, switch_count;
         
@@ -443,8 +470,13 @@ public:
         
         channel_switch_type(uint8_t mode, uint8_t channel, uint8_t count)
         : switch_mode(mode), new_channel(channel), switch_count(count) { }
+
+        static channel_switch_type from_option(const option &opt);
     };
     
+    /**
+     * The type used to store the Quiet option data.
+     */
     struct quiet_type {
         uint8_t quiet_count, quiet_period;
         uint16_t quiet_duration, quiet_offset;
@@ -455,8 +487,13 @@ public:
           uint16_t offset)
         : quiet_count(count), quiet_period(period), 
         quiet_duration(duration), quiet_offset(offset) {}
+
+        static quiet_type from_option(const option &opt);
     };
-    
+
+    /**
+     * The type used to store the BSS Load option data.
+     */
     struct bss_load_type {
         uint16_t station_count;
         uint16_t available_capacity;
@@ -468,22 +505,48 @@ public:
           uint16_t capacity) 
         : station_count(count), available_capacity(capacity),
         channel_utilization(utilization) {}
+
+        static bss_load_type from_option(const option &opt);
     };
     
+    /**
+     * The type used to store the TIM option data.
+     */
     struct tim_type {
-        typedef std::vector<uint8_t> container_type;
-        
         uint8_t dtim_count, dtim_period, bitmap_control;
-        container_type partial_virtual_bitmap;
+        byte_array partial_virtual_bitmap;
         
         tim_type() {}
         
         tim_type(uint8_t count, uint8_t period, uint8_t control,
-          const container_type &bitmap) 
+          const byte_array &bitmap) 
         : dtim_count(count), dtim_period(period), bitmap_control(control),
         partial_virtual_bitmap(bitmap) {}
+
+        static tim_type from_option(const option &opt);
     };
 
+    /**
+     * The type used to store the Vendor Specific option data.
+     */
+    struct vendor_specific_type {
+        typedef HWAddress<3> oui_type;
+
+        oui_type oui;
+        byte_array data;
+
+        vendor_specific_type(const oui_type &oui = oui_type(),
+            const byte_array &data = byte_array())
+        : oui(oui), data(data) { }
+
+        static vendor_specific_type from_bytes(const uint8_t *buffer, uint32_t sz);
+    };
+    
+    /**
+     * The type used to store the QOS capability tagged option data.
+     */
+    typedef uint8_t qos_capability_type;
+
     /**
      * \brief Getter for the second address.
      *
@@ -527,7 +590,7 @@ public:
     /**
      * \brief Getter for the fourth address.
      *
-     * \return address_type containing the fourth address.
+     * \return The stored fourth address.
      */
     const address_type &addr4() const { return _addr4; }
 
@@ -569,9 +632,9 @@ public:
     // Option setter helpers
 
     /**
-     * \brief Helper method to set the ssid.
+     * \brief Helper method to set the SSID.
      *
-     * \param new_ssid The ssid to be set.
+     * \param new_ssid The SSID to be set.
      */
     void ssid(const std::string &new_ssid);
 
@@ -583,28 +646,28 @@ public:
     void rsn_information(const RSNInformation& info);
 
     /**
-     * \brief Helper method to set the supported rates.
+     * \brief Helper method to set the supported rates option.
      *
      * \param new_rates The new rates to be set.
      */
     void supported_rates(const rates_type &new_rates);
 
     /**
-     * \brief Helper method to set the extended supported rates.
+     * \brief Helper method to set the extended supported rates option.
      *
      * \param new_rates The new rates to be set.
      */
     void extended_supported_rates(const rates_type &new_rates);
 
     /**
-     * \brief Helper method to set the QoS capabilities.
+     * \brief Helper method to set the QoS capabilities option.
      *
      * \param new_qos_capabilities uint8_t with the capabilities.
      */
-    void qos_capability(uint8_t new_qos_capability);
+    void qos_capability(qos_capability_type new_qos_capability);
 
     /**
-     * \brief Helper method to set the power capabilities.
+     * \brief Helper method to set the power capabilities option.
      *
      * \param min_power uint8_t indicating the minimum transmiting power capability.
      * \param max_power uint8_t indicating the maximum transmiting power capability.
@@ -612,7 +675,11 @@ public:
     void power_capability(uint8_t min_power, uint8_t max_power);
 
     /**
-     * \brief Helper method to set the supported channels.
+     * \brief Helper method to set the supported channels option.
+     * 
+     * Each element in the provided vector should be a tuple 
+     * (First channel number, number of channels), as defined in the
+     * standard.
      *
      * \param new_channels A list of channels to be set.
      */
@@ -629,35 +696,35 @@ public:
     void edca_parameter_set(uint32_t ac_be, uint32_t ac_bk, uint32_t ac_vi, uint32_t ac_vo);
 
     /**
-     * \brief Helper method to set the Request Information element.
+     * \brief Helper method to set the Request Information element tagged option.
      *
-     * \param elements A list of elements.
+     * \param elements The new list of elements.
      */
     void request_information(const request_info_type elements);
 
     /**
-     * \brief Helper method to set the FH parameter.
+     * \brief Helper method to set the FH parameter set tagged option.
      *
-     * \param fh_params the fh parameter set.
+     * \param fh_params The new FH parameter set value.
      */
-    void fh_parameter_set(fh_params_set fh_params);
+    void fh_parameter_set(const fh_params_set &fh_params);
 
     /**
-     * \brief Helper method to set the DS parameter.
+     * \brief Helper method to set the DS parameter tagged option.
      *
-     * \param current_channel uint8_t with the value of the current_channel field.
+     * \param current_channel The access point's new current channel.
      */
     void ds_parameter_set(uint8_t current_channel);
 
     /**
-     * \brief Helper method to set the CF parameter.
+     * \brief Helper method to set the CF parameter set tagged option.
      *
-     * \param params the CF parammeters to be set.
+     * \param params The new CF parameter set value.
      */
-    void cf_parameter_set(cf_params_set params);
+    void cf_parameter_set(const cf_params_set &params);
 
     /**
-     * \brief Helper method to set the IBSS parameter.
+     * \brief Helper method to set the IBSS parameter set tagged option.
      *
      * \param atim_window uint16_t with the value of the ATIM window field.
      */
@@ -678,17 +745,17 @@ public:
     void country(const country_params &params);
 
     /**
-     * \brief Helper method to set the FH parameters.
+     * \brief Helper method to set the FH parameters set tagged option.
      *
-     * \param prime_radix uint8_t with the value of the prime radix field.
-     * \param number_channels uint8_t with the value of the number channels field.
+     * \param prime_radix The value of the prime radix field.
+     * \param number_channels The value of the number channels field.
      */
     void fh_parameters(uint8_t prime_radix, uint8_t number_channels);
 
     /**
-     * \brief Helper method to set the FH pattern table.
+     * \brief Helper method to set the FH pattern table tagged option.
      *
-     * \param params The data to be used for this fh_pattern_table option.
+     * \param params The data to be used for this FH pattern table option.
      */
     void fh_pattern_table(const fh_pattern_type &params);
 
@@ -748,6 +815,13 @@ public:
      * \brief text The challenge text to be added.
      */
     void challenge_text(const std::string &text);
+
+    /**
+     * \brief Helper method to add a Vendor Specific tagged option.
+     *
+     * \brief text The option to be added.
+     */
+    void vendor_specific(const vendor_specific_type &data);
     
     // Option searching helpers
     
@@ -763,12 +837,12 @@ public:
     RSNInformation rsn_information();
     
     /**
-     * \brief Helper method to search for this PDU's ssid.
+     * \brief Helper method to search for this PDU's SSID.
      * 
      * An option_not_found exception is thrown if the option has not 
      * been set.
      * 
-     * \return std::string containing the ssid.
+     * \return std::string containing the SSID.
      */
     std::string ssid() const;
 
@@ -800,7 +874,7 @@ public:
      * 
      * \return uint8_t containing the QOS capability.
      */
-    uint8_t qos_capability() const;
+    qos_capability_type qos_capability() const;
 
     /**
      * \brief Helper method to get the power capability.
@@ -815,6 +889,10 @@ public:
     /**
      * \brief Helper method to get the supported channels.
      *
+     * Each element in the provided vector is a tuple 
+     * (First channel number, number of channels), as defined in the
+     * standard.
+     * 
      * An option_not_found exception is thrown if the option has not 
      * been set.
      * 
@@ -843,14 +921,25 @@ public:
     fh_params_set fh_parameter_set() const;
     
     /**
-     * \brief Helper method to get the ds parameter set.
+     * \brief Helper method to get the DSSS parameter set.
+     * 
+     * An option_not_found exception is thrown if the option has not 
+     * been set.
+     * 
+     * \return The access point's current channel.
+     */
+    uint8_t ds_parameter_set() const;
+
+    /**
+     * \brief Helper method to get the CF parameter set.
      *
      * An option_not_found exception is thrown if the option has not 
      * been set.
      * 
-     * \return uint8_t containing the ds parameter set.
+     * \return The CF parameter set.
      */
-    uint8_t ds_parameter_set() const;
+    cf_params_set cf_parameter_set() const;
+    
     
     /**
      * \brief Helper method to get the ibss parameter set.
@@ -982,6 +1071,16 @@ public:
      */
     std::string challenge_text() const;
     
+    /**
+     * \brief Helper method to get a Vendor Specific option.
+     *
+     * An option_not_found exception is thrown if the option has not 
+     * been set.
+     * 
+     * \return vendor_specific_type containing the option value.
+     */
+    vendor_specific_type vendor_specific() const;
+
     // ************************
 
     /**
@@ -1044,6 +1143,14 @@ protected:
 private:
     static uint8_t *serialize_rates(const rates_type &rates);
     static rates_type deserialize_rates(const option *option);
+    
+    template<typename T>
+    T search_and_convert(OptionTypes opt_type) const {
+        const option *opt = search_option(opt_type);
+        if(!opt)
+            throw option_not_found();
+        return opt->to<T>();
+    }
 
     ExtendedHeader _ext_header;
     address_type _addr4;
