@@ -169,7 +169,7 @@ namespace Tins {
         /**
          * The IP options type.
          */
-        typedef PDUOption<option_identifier> option;
+        typedef PDUOption<option_identifier, IP> option;
 
         /**
          * The type of the security option.
@@ -184,6 +184,8 @@ namespace Tins {
             : security(sec), compartments(comp), 
               handling_restrictions(hand_res), transmission_control(tcc) 
               {}
+            
+            static security_type from_option(const option &opt);
         };
         
         /**
@@ -198,6 +200,8 @@ namespace Tins {
             generic_route_option_type(uint8_t ptr = 0, 
               routes_type rts = routes_type())
             : pointer(ptr), routes(rts) {}
+            
+            static generic_route_option_type from_option(const option &opt);
         };
         
         /**
@@ -333,13 +337,6 @@ namespace Tins {
 
         /* Setters */
 
-        /**
-         * \brief Setter for the header length field.
-         *
-         * \param new_head_len The new header length.
-         */
-        void head_len(small_uint<4> new_head_len);
-
         /**
          * \brief Setter for the type of service field.
          *
@@ -347,13 +344,6 @@ namespace Tins {
          */
         void tos(uint8_t new_tos);
 
-        /**
-         * \brief Setter for the total length field.
-         *
-         * \param new_tot_len The new total length.
-         */
-        void tot_len(uint16_t new_tot_len);
-
         /**
          * \brief Setter for the id field.
          *
@@ -378,6 +368,17 @@ namespace Tins {
         /**
          * \brief Setter for the protocol field.
          *
+         * Note that this protocol will be overwritten using the 
+         * inner_pdu's protocol type during serialization unless the IP 
+         * datagram is fragmented. 
+         * 
+         * If the packet is fragmented and was originally sniffed, the
+         * original protocol type will be kept when serialized.
+         * 
+         * If this packet has been crafted manually and the inner_pdu
+         * is, for example, a RawPDU, then setting the protocol yourself
+         * is necessary.
+         * 
          * \param new_protocol The new protocol.
          */
         void protocol(uint8_t new_protocol);
@@ -582,6 +583,13 @@ namespace Tins {
          */
         PDU *recv_response(PacketSender &sender, const NetworkInterface &);
 
+        /**
+         * Indicates whether this PDU is fragmented.
+         *
+         * \return true if this PDU is fragmented, false otherwise.
+         */
+        bool is_fragmented() const;
+
         /**
          * \brief Getter for the PDU's type.
          * \sa PDU::pdu_type
@@ -618,6 +626,10 @@ namespace Tins {
             /*The options start here. */
         } TINS_END_PACK;
 
+
+        void head_len(small_uint<4> new_head_len);
+        void tot_len(uint16_t new_tot_len);
+
         void prepare_for_serialize(const PDU *parent);
         void internal_add_option(const option &option);
         void init_ip_fields();
