@@ -27,10 +27,15 @@
  *
  */
 
-#include <openssl/evp.h>
-#include <openssl/hmac.h>
-#include <openssl/aes.h>
 #include "crypto.h"
+
+#ifdef HAVE_DOT11
+
+#ifdef HAVE_WPA2_DECRYPTION
+    #include <openssl/evp.h>
+    #include <openssl/hmac.h>
+    #include <openssl/aes.h>
+#endif // HAVE_WPA2_DECRYPTION
 #include "dot11/dot11_data.h"
 #include "dot11/dot11_beacon.h"
 
@@ -540,3 +545,5 @@ bool WPA2Decrypter::decrypt(PDU &pdu) {
 #endif // HAVE_WPA2_DECRYPTION
 } // namespace Crypto
 } // namespace Tins
+
+#endif // HAVE_DOT11
\ No newline at end of file
