@@ -27,8 +27,10 @@
  *
  */   
 
-#include <cstring>
 #include "dot11/dot11_mgmt.h"
+#ifdef HAVE_DOT11
+
+#include <cstring>
 #include "rsn_information.h"
 
 namespace Tins {
@@ -147,7 +149,7 @@ void Dot11ManagementFrame::extended_supported_rates(const rates_type &new_rates)
     delete[] buffer;
 }
 
-void Dot11ManagementFrame::qos_capability(uint8_t new_qos_capability) {
+void Dot11ManagementFrame::qos_capability(qos_capability_type new_qos_capability) {
     add_tagged_option(QOS_CAPABILITY, 1, &new_qos_capability);
 }
 
@@ -159,14 +161,13 @@ void Dot11ManagementFrame::power_capability(uint8_t min_power, uint8_t max_power
 }
 
 void Dot11ManagementFrame::supported_channels(const channels_type &new_channels) {
-    uint8_t* buffer = new uint8_t[new_channels.size() * 2];
-    uint8_t* ptr = buffer;
+    std::vector<uint8_t> buffer(new_channels.size() * 2);
+    uint8_t* ptr = &buffer[0];
     for(channels_type::const_iterator it = new_channels.begin(); it != new_channels.end(); ++it) {
         *(ptr++) = it->first;
         *(ptr++) = it->second;
     }
-    add_tagged_option(SUPPORTED_CHANNELS, new_channels.size() * 2, buffer);
-    delete[] buffer;
+    add_tagged_option(SUPPORTED_CHANNELS, buffer.size(), &buffer[0]);
 }
 
 void Dot11ManagementFrame::edca_parameter_set(uint32_t ac_be, uint32_t ac_bk, uint32_t ac_vi, uint32_t ac_vo) {
@@ -182,19 +183,17 @@ void Dot11ManagementFrame::edca_parameter_set(uint32_t ac_be, uint32_t ac_bk, ui
 }
 
 void Dot11ManagementFrame::request_information(const request_info_type elements) {
-    uint8_t *buffer = new uint8_t[elements.size()], *ptr = buffer;
-    for (request_info_type::const_iterator it = elements.begin(); it != elements.end(); ++it)
-        *(ptr++) = *it;
-    add_tagged_option(REQUEST_INFORMATION, elements.size(), buffer);
-    delete[] buffer;
+    add_tagged_option(REQUEST_INFORMATION, elements.size(), &elements[0]);
 }
 
-void Dot11ManagementFrame::fh_parameter_set(fh_params_set fh_params) {
-    fh_params.dwell_time = Endian::host_to_le(fh_params.dwell_time);
-    fh_params.hop_set = fh_params.hop_set;
-    fh_params.hop_pattern = fh_params.hop_pattern;
-    fh_params.hop_index = fh_params.hop_index;
-    add_tagged_option(FH_SET, sizeof(fh_params_set), (uint8_t*)&fh_params);
+void Dot11ManagementFrame::fh_parameter_set(const fh_params_set &fh_params) {
+    uint8_t data[5];
+    uint16_t dwell = Endian::host_to_le(fh_params.dwell_time);
+    std::memcpy(data, &dwell, sizeof(dwell));
+    data[2] = fh_params.hop_set;
+    data[3] = fh_params.hop_pattern;
+    data[4] = fh_params.hop_index;
+    add_tagged_option(FH_SET, sizeof(data), data);
 
 }
 
@@ -202,12 +201,15 @@ void Dot11ManagementFrame::ds_parameter_set(uint8_t current_channel) {
     add_tagged_option(DS_SET, 1, &current_channel);
 }
 
-void Dot11ManagementFrame::cf_parameter_set(cf_params_set params) {
-    params.cfp_count = params.cfp_count;
-    params.cfp_period = params.cfp_period;
-    params.cfp_max_duration = Endian::host_to_le(params.cfp_max_duration);
-    params.cfp_dur_remaining = Endian::host_to_le(params.cfp_dur_remaining);
-    add_tagged_option(CF_SET, sizeof(params), (uint8_t*)&params);
+void Dot11ManagementFrame::cf_parameter_set(const cf_params_set &params) {
+    uint8_t data[6];
+    data[0] = params.cfp_count;
+    data[1] = params.cfp_period;
+    uint16_t dummy = Endian::host_to_le(params.cfp_max_duration);
+    std::memcpy(data + 2, &dummy, sizeof(uint16_t));
+    dummy = Endian::host_to_le(params.cfp_dur_remaining);
+    std::memcpy(data + 4, &dummy, sizeof(uint16_t));
+    add_tagged_option(CF_SET, sizeof(data), data);
 }
 
 void Dot11ManagementFrame::ibss_parameter_set(uint16_t atim_window) {
@@ -266,7 +268,7 @@ void Dot11ManagementFrame::fh_pattern_table(const fh_pattern_type &params) {
     *(ptr++) = params.number_of_sets;
     *(ptr++) = params.modulus;
     *(ptr++) = params.offset;
-    fh_pattern_type::container_type::const_iterator it(params.random_table.begin());
+    byte_array::const_iterator it(params.random_table.begin());
     for(; it != params.random_table.end(); ++it)
         *(ptr++) = *it;
     add_tagged_option(HOPPING_PATTERN_TABLE, data.size(), &data[0]);
@@ -355,13 +357,20 @@ void Dot11ManagementFrame::challenge_text(const std::string &text) {
     );
 }
 
+void Dot11ManagementFrame::vendor_specific(const vendor_specific_type &data) {
+    byte_array buffer(3 + data.data.size());
+    std::copy(
+        data.data.begin(),
+        data.data.end(),
+        data.oui.copy(buffer.begin())
+    );
+    add_tagged_option(VENDOR_SPECIFIC, buffer.size(), &buffer[0]);
+}
+
 // Getters
 
 RSNInformation Dot11ManagementFrame::rsn_information() {
-    const Dot11::option *option = search_option(RSN);
-    if(!option || option->data_size() < (sizeof(uint16_t) << 1) + sizeof(uint32_t))
-        throw option_not_found();
-    return RSNInformation(option->data_ptr(), option->data_size());
+    return search_and_convert<RSNInformation>(RSN);
 }
 
 std::string Dot11ManagementFrame::ssid() const {
@@ -375,89 +384,143 @@ std::string Dot11ManagementFrame::ssid() const {
 }
 
 Dot11ManagementFrame::rates_type Dot11ManagementFrame::supported_rates() const {
-    const Dot11::option *option = search_option(SUPPORTED_RATES);
-    if(!option || option->data_size() == 0)
-        throw option_not_found();
-    return deserialize_rates(option);
+    return search_and_convert<rates_type>(SUPPORTED_RATES);
 }
 
 Dot11ManagementFrame::rates_type Dot11ManagementFrame::extended_supported_rates() const {
-    const Dot11::option *option = search_option(EXT_SUPPORTED_RATES);
-    if(!option || option->data_size() == 0)
-        throw option_not_found();
-    return deserialize_rates(option);
+    return search_and_convert<rates_type>(EXT_SUPPORTED_RATES);
 }
 
-uint8_t Dot11ManagementFrame::qos_capability() const {
-    const Dot11::option *option = search_option(QOS_CAPABILITY);
-    if(!option || option->data_size() != 1)
-        throw option_not_found();
-    return *option->data_ptr();
+Dot11ManagementFrame::qos_capability_type Dot11ManagementFrame::qos_capability() const {
+    return search_and_convert<uint8_t>(QOS_CAPABILITY);
 }
 
 std::pair<uint8_t, uint8_t> Dot11ManagementFrame::power_capability() const {
-    const Dot11::option *option = search_option(POWER_CAPABILITY);
-    if(!option || option->data_size() != 2)
-        throw option_not_found();
-    return std::make_pair(*option->data_ptr(), *(option->data_ptr() + 1));
+    return search_and_convert<std::pair<uint8_t, uint8_t> >(POWER_CAPABILITY);
 }
 
 Dot11ManagementFrame::channels_type Dot11ManagementFrame::supported_channels() const {
-    const Dot11::option *option = search_option(SUPPORTED_CHANNELS);
-    // We need a multiple of two
-    if(!option || ((option->data_size() & 0x1) == 1))
-        throw option_not_found();
-    channels_type output;
-    const uint8_t *ptr = option->data_ptr(), *end = ptr + option->data_size();
-    while(ptr != end) {
-        uint8_t first = *(ptr++);
-        output.push_back(std::make_pair(first, *(ptr++)));
-    }
-    return output;
+    return search_and_convert<channels_type>(SUPPORTED_CHANNELS);
 }
 
 Dot11ManagementFrame::request_info_type Dot11ManagementFrame::request_information() const {
-    const Dot11::option *option = search_option(REQUEST_INFORMATION);
-    if(!option || option->data_size() == 0)
-        throw option_not_found();
-    request_info_type output;
-    const uint8_t *ptr = option->data_ptr(), *end = ptr + option->data_size();
-    output.assign(ptr, end);
-    return output;
+    return search_and_convert<request_info_type>(REQUEST_INFORMATION);
 }
 
 Dot11ManagementFrame::fh_params_set Dot11ManagementFrame::fh_parameter_set() const {
-    const Dot11::option *option = search_option(FH_SET);
-    if(!option || option->data_size() != sizeof(fh_params_set))
-        throw option_not_found();
-    fh_params_set output = *reinterpret_cast<const fh_params_set*>(option->data_ptr());
-    output.dwell_time = Endian::le_to_host(output.dwell_time);
-    output.hop_set = output.hop_set;
-    output.hop_pattern = output.hop_pattern;
-    output.hop_index = output.hop_index;
-    return output;
+    return search_and_convert<fh_params_set>(FH_SET);
 }
 
 uint8_t Dot11ManagementFrame::ds_parameter_set() const {
-    const Dot11::option *option = search_option(DS_SET);
-    if(!option || option->data_size() != sizeof(uint8_t))
-        throw option_not_found();
-    return *option->data_ptr();
+    return search_and_convert<uint8_t>(DS_SET);
+}
+
+Dot11ManagementFrame::cf_params_set Dot11ManagementFrame::cf_parameter_set() const {
+    return search_and_convert<cf_params_set>(CF_SET);
 }
 
 uint16_t Dot11ManagementFrame::ibss_parameter_set() const {
-    const Dot11::option *option = search_option(IBSS_SET);
-    if(!option || option->data_size() != sizeof(uint16_t))
-        throw option_not_found();
-    return Endian::le_to_host(*reinterpret_cast<const uint16_t*>(option->data_ptr()));
+    return search_and_convert<uint16_t>(IBSS_SET);
 }
 
 Dot11ManagementFrame::ibss_dfs_params Dot11ManagementFrame::ibss_dfs() const {
-    const Dot11::option *option = search_option(IBSS_DFS);
-    if(!option || option->data_size() < ibss_dfs_params::minimum_size)
+    return search_and_convert<ibss_dfs_params>(IBSS_DFS);
+}
+
+Dot11ManagementFrame::country_params Dot11ManagementFrame::country() const {
+    return search_and_convert<country_params>(COUNTRY);
+}
+
+std::pair<uint8_t, uint8_t> Dot11ManagementFrame::fh_parameters() const {
+    return search_and_convert<std::pair<uint8_t, uint8_t> >(HOPPING_PATTERN_PARAMS);
+}
+
+Dot11ManagementFrame::fh_pattern_type Dot11ManagementFrame::fh_pattern_table() const {
+    return search_and_convert<fh_pattern_type>(HOPPING_PATTERN_TABLE);
+}
+
+uint8_t Dot11ManagementFrame::power_constraint() const {
+    return search_and_convert<uint8_t>(POWER_CONSTRAINT);
+}
+
+Dot11ManagementFrame::channel_switch_type Dot11ManagementFrame::channel_switch() const {
+    return search_and_convert<channel_switch_type>(CHANNEL_SWITCH);
+}
+
+Dot11ManagementFrame::quiet_type Dot11ManagementFrame::quiet() const {
+    return search_and_convert<quiet_type>(QUIET);
+}
+
+std::pair<uint8_t, uint8_t> Dot11ManagementFrame::tpc_report() const {
+    return search_and_convert<std::pair<uint8_t, uint8_t> >(TPC_REPORT);
+}
+
+uint8_t Dot11ManagementFrame::erp_information() const {
+    return search_and_convert<uint8_t>(ERP_INFORMATION);
+}
+
+Dot11ManagementFrame::bss_load_type Dot11ManagementFrame::bss_load() const {
+    return search_and_convert<bss_load_type>(BSS_LOAD);
+}
+
+Dot11ManagementFrame::tim_type Dot11ManagementFrame::tim() const {
+    return search_and_convert<tim_type>(TIM);
+}
+
+std::string Dot11ManagementFrame::challenge_text() const {
+    return search_and_convert<std::string>(CHALLENGE_TEXT);
+}
+
+Dot11ManagementFrame::vendor_specific_type Dot11ManagementFrame::vendor_specific() const {
+    const Dot11::option *option = search_option(VENDOR_SPECIFIC);
+    if(!option || option->data_size() < 3)
         throw option_not_found();
+    return vendor_specific_type::from_bytes(option->data_ptr(), option->data_size());
+}
+
+Dot11ManagementFrame::vendor_specific_type 
+  Dot11ManagementFrame::vendor_specific_type::from_bytes(const uint8_t *buffer, uint32_t sz) 
+{
+    if(sz < 3)
+        throw malformed_option();
+    return vendor_specific_type(
+        buffer, 
+        byte_array(buffer + 3, buffer + sz)
+    );
+}
+
+// Options
+
+Dot11ManagementFrame::fh_params_set Dot11ManagementFrame::fh_params_set::from_option(const option &opt)
+{
+    if(opt.data_size() != 5)
+        throw malformed_option();
+    fh_params_set output;
+    output.dwell_time = Endian::le_to_host(*(uint16_t*)opt.data_ptr());
+    output.hop_set = opt.data_ptr()[2];
+    output.hop_pattern = opt.data_ptr()[3];
+    output.hop_index = opt.data_ptr()[4];
+    return output;
+}
+
+Dot11ManagementFrame::cf_params_set Dot11ManagementFrame::cf_params_set::from_option(const option &opt) 
+{
+    if(opt.data_size() != 6)
+        throw malformed_option();
+    cf_params_set output;
+    output.cfp_count = *opt.data_ptr();
+    output.cfp_period = opt.data_ptr()[1];
+    output.cfp_max_duration = Endian::le_to_host(*(uint16_t*)&opt.data_ptr()[2]);
+    output.cfp_dur_remaining = Endian::le_to_host(*(uint16_t*)&opt.data_ptr()[4]);
+    return output;
+}
+
+Dot11ManagementFrame::ibss_dfs_params Dot11ManagementFrame::ibss_dfs_params::from_option(const option &opt)
+{
+    if(opt.data_size() < ibss_dfs_params::minimum_size)
+        throw malformed_option();
     ibss_dfs_params output;
-    const uint8_t *ptr = option->data_ptr(), *end = ptr + option->data_size();
+    const uint8_t *ptr = opt.data_ptr(), *end = ptr + opt.data_size();
     output.dfs_owner = ptr;
     ptr += output.dfs_owner.size();
     output.recovery_interval = *(ptr++);
@@ -470,12 +533,12 @@ Dot11ManagementFrame::ibss_dfs_params Dot11ManagementFrame::ibss_dfs() const {
     return output;
 }
 
-Dot11ManagementFrame::country_params Dot11ManagementFrame::country() const {
-    const Dot11::option *option = search_option(COUNTRY);
-    if(!option || option->data_size() < country_params::minimum_size)
-        throw option_not_found();
+Dot11ManagementFrame::country_params Dot11ManagementFrame::country_params::from_option(const option &opt)
+{
+    if(opt.data_size() < country_params::minimum_size)
+        throw malformed_option();
     country_params output;
-    const uint8_t *ptr = option->data_ptr(), *end = ptr + option->data_size();
+    const uint8_t *ptr = opt.data_ptr(), *end = ptr + opt.data_size();
     std::copy(ptr, ptr + 3, std::back_inserter(output.country));
     ptr += output.country.size();
     while(end - ptr >= 3) {
@@ -484,25 +547,16 @@ Dot11ManagementFrame::country_params Dot11ManagementFrame::country() const {
         output.max_transmit_power.push_back(*(ptr++));
     }
     if(ptr != end)
-        throw option_not_found();
-    return output;
+        throw malformed_option();
+    return output; 
 }
 
-std::pair<uint8_t, uint8_t> Dot11ManagementFrame::fh_parameters() const {
-    const Dot11::option *option = search_option(HOPPING_PATTERN_PARAMS);
-    if(!option || option->data_size() != sizeof(uint8_t) * 2)
-        throw option_not_found();
-    const uint8_t *ptr = option->data_ptr();
-    uint8_t first = *(ptr++);
-    return std::make_pair(first, *ptr);
-}
-
-Dot11ManagementFrame::fh_pattern_type Dot11ManagementFrame::fh_pattern_table() const {
-    const Dot11::option *option = search_option(HOPPING_PATTERN_TABLE);
-    if(!option || option->data_size() < fh_pattern_type::minimum_size)
-        throw option_not_found();
+Dot11ManagementFrame::fh_pattern_type Dot11ManagementFrame::fh_pattern_type::from_option(const option &opt)
+{
+    if(opt.data_size() < fh_pattern_type::minimum_size)
+        throw malformed_option();
     fh_pattern_type output;
-    const uint8_t *ptr = option->data_ptr(), *end = ptr + option->data_size();
+    const uint8_t *ptr = opt.data_ptr(), *end = ptr + opt.data_size();
     
     output.flag = *(ptr++);
     output.number_of_sets = *(ptr++);
@@ -513,18 +567,11 @@ Dot11ManagementFrame::fh_pattern_type Dot11ManagementFrame::fh_pattern_table() c
     return output;
 }
 
-uint8_t Dot11ManagementFrame::power_constraint() const {
-    const Dot11::option *option = search_option(POWER_CONSTRAINT);
-    if(!option || option->data_size() != 1)
-        throw option_not_found();
-    return *option->data_ptr();
-}
-
-Dot11ManagementFrame::channel_switch_type Dot11ManagementFrame::channel_switch() const {
-    const Dot11::option *option = search_option(CHANNEL_SWITCH);
-    if(!option || option->data_size() != sizeof(uint8_t) * 3)
-        throw option_not_found();
-    const uint8_t *ptr = option->data_ptr();
+Dot11ManagementFrame::channel_switch_type Dot11ManagementFrame::channel_switch_type::from_option(const option &opt)
+{
+    if(opt.data_size() != sizeof(uint8_t) * 3)
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr();
     channel_switch_type output;
     output.switch_mode = *(ptr++);
     output.new_channel = *(ptr++);
@@ -532,11 +579,11 @@ Dot11ManagementFrame::channel_switch_type Dot11ManagementFrame::channel_switch()
     return output;
 }
 
-Dot11ManagementFrame::quiet_type Dot11ManagementFrame::quiet() const {
-    const Dot11::option *option = search_option(QUIET);
-    if(!option || option->data_size() != (sizeof(uint8_t) * 2 + sizeof(uint16_t) * 2))
-        throw option_not_found();
-    const uint8_t *ptr = option->data_ptr();
+Dot11ManagementFrame::quiet_type Dot11ManagementFrame::quiet_type::from_option(const option &opt)
+{
+    if(opt.data_size() != (sizeof(uint8_t) * 2 + sizeof(uint16_t) * 2))
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr();
     quiet_type output;
     
     output.quiet_count = *(ptr++);
@@ -547,40 +594,24 @@ Dot11ManagementFrame::quiet_type Dot11ManagementFrame::quiet() const {
     return output;
 }
 
-std::pair<uint8_t, uint8_t> Dot11ManagementFrame::tpc_report() const {
-    const Dot11::option *option = search_option(TPC_REPORT);
-    if(!option || option->data_size() != sizeof(uint8_t) * 2)
-        throw option_not_found();
-    const uint8_t *ptr = option->data_ptr();
-    uint8_t first = *(ptr++);
-    return std::make_pair(first, *ptr);
-}
-
-uint8_t Dot11ManagementFrame::erp_information() const {
-    const Dot11::option *option = search_option(ERP_INFORMATION);
-    if(!option || option->data_size() != sizeof(uint8_t))
-        throw option_not_found();
-    return *option->data_ptr();
-}
-
-Dot11ManagementFrame::bss_load_type Dot11ManagementFrame::bss_load() const {
-    const Dot11::option *option = search_option(BSS_LOAD);
-    if(!option || option->data_size() != sizeof(uint8_t) + 2 * sizeof(uint16_t))
-        throw option_not_found();
+Dot11ManagementFrame::bss_load_type Dot11ManagementFrame::bss_load_type::from_option(const option &opt)
+{
+    if(opt.data_size() != sizeof(uint8_t) + 2 * sizeof(uint16_t))
+        throw malformed_option();
     bss_load_type output;
     
-    const uint8_t *ptr = option->data_ptr();
+    const uint8_t *ptr = opt.data_ptr();
     output.station_count = Endian::le_to_host(*(uint16_t*)ptr);
     output.channel_utilization = ptr[2];
     output.available_capacity = Endian::le_to_host(*(uint16_t*)(ptr + 3));
     return output;
 }
 
-Dot11ManagementFrame::tim_type Dot11ManagementFrame::tim() const {
-    const Dot11::option *option = search_option(TIM);
-    if(!option || option->data_size() < 4 * sizeof(uint8_t))
-        throw option_not_found();
-    const uint8_t *ptr = option->data_ptr(), *end = ptr + option->data_size();
+Dot11ManagementFrame::tim_type Dot11ManagementFrame::tim_type::from_option(const option &opt)
+{
+    if(opt.data_size() < 4 * sizeof(uint8_t))
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr(), *end = ptr + opt.data_size();
     tim_type output;
     
     output.dtim_count = *(ptr++);
@@ -590,12 +621,6 @@ Dot11ManagementFrame::tim_type Dot11ManagementFrame::tim() const {
     output.partial_virtual_bitmap.assign(ptr, end);
     return output;
 }
+} // namespace Tins
 
-std::string Dot11ManagementFrame::challenge_text() const {
-    const Dot11::option *option = search_option(CHALLENGE_TEXT);
-    if(!option || option->data_size() == 0)
-        throw option_not_found();
-    return std::string(option->data_ptr(), option->data_ptr() + option->data_size());
-}
-
-} // namespace Tins
\ No newline at end of file
+#endif // HAVE_DOT11
