@@ -225,7 +225,7 @@ void ICMPv6::write_serialization(uint8_t *buffer, uint32_t total_sz, const PDU *
         buffer = write_option(*it, buffer);
     }
     if(!_header.cksum) {
-        const Tins::IPv6 *ipv6 = dynamic_cast<const Tins::IPv6*>(parent);
+        const Tins::IPv6 *ipv6 = tins_cast<const Tins::IPv6*>(parent);
         if(ipv6) {
             uint32_t checksum = Utils::pseudoheader_checksum(
                                     ipv6->src_addr(),  
@@ -294,39 +294,57 @@ void ICMPv6::prefix_info(prefix_info_type info) {
     );
 }
 
-void ICMPv6::redirect_header(PDU::serialization_type data) {
-    // Reserved fields
-    data.insert(data.begin(), 6, 0);
-    // Padding(if necessary)
-    uint8_t padding = 8 - (data.size() + sizeof(uint8_t) * 2) % 8;
-    if(padding == 8)
-        padding = 0;
-    data.insert(data.end(), padding, 0);
+void ICMPv6::redirect_header(const byte_array& data) {
     add_option(option(REDIRECT_HEADER, data.begin(), data.end()));
 }
 
-void ICMPv6::mtu(uint32_t value) {
+void ICMPv6::mtu(const mtu_type& value) {
     uint8_t buffer[sizeof(uint16_t) + sizeof(uint32_t)] = {0};
-    *((uint32_t*)(buffer + sizeof(uint16_t))) = Endian::host_to_be(value);
+    const uint16_t u16_tmp = value.first; 
+    const uint32_t u32_tmp = value.second; 
+    buffer[0] = u16_tmp >> 8;
+    buffer[1] = u16_tmp & 0xff;
+
+    buffer[2] = u32_tmp >> 24;
+    buffer[3] = u32_tmp >> 16;
+    buffer[4] = u32_tmp >> 8;
+    buffer[5] = u32_tmp & 0xff;
     add_option(option(MTU, sizeof(buffer), buffer));
 }
 
-void ICMPv6::shortcut_limit(uint8_t value) {
+void ICMPv6::shortcut_limit(const shortcut_limit_type &value) {
     uint8_t buffer[sizeof(uint16_t) + sizeof(uint32_t)] = {0};
-    buffer[0] = value;
+    const uint32_t u32_tmp = value.reserved2; 
+    buffer[0] = value.limit;
+    buffer[1] = value.reserved1;
+    buffer[2] = u32_tmp >> 24;
+    buffer[3] = u32_tmp >> 16;
+    buffer[4] = u32_tmp >> 8;
+    buffer[5] = u32_tmp & 0xff;
     add_option(option(NBMA_SHORT_LIMIT, sizeof(buffer), buffer));
 }
 
-void ICMPv6::new_advert_interval(uint32_t value) {
+void ICMPv6::new_advert_interval(const new_advert_interval_type &value) {
     uint8_t buffer[sizeof(uint16_t) + sizeof(uint32_t)] = {0};
-    *((uint32_t*)(buffer + sizeof(uint16_t))) = Endian::host_to_be(value);
+    const uint16_t u16_tmp = value.reserved;
+    const uint32_t u32_tmp = value.interval;
+    buffer[0] = u16_tmp >> 8;
+    buffer[1] = u16_tmp & 0xff;
+
+    buffer[2] = u32_tmp >> 24;
+    buffer[3] = u32_tmp >> 16;
+    buffer[4] = u32_tmp >> 8;
+    buffer[5] = u32_tmp & 0xff;
     add_option(option(ADVERT_INTERVAL, sizeof(buffer), buffer));
 }
 
 void ICMPv6::new_home_agent_info(const new_ha_info_type &value) {
+    if(value.size() != 3)
+        throw malformed_option();
     uint8_t buffer[sizeof(uint16_t) + sizeof(uint32_t)] = {0};
-    *((uint16_t*)(buffer + sizeof(uint16_t))) = Endian::host_to_be(value.first);
-    *((uint16_t*)(buffer + sizeof(uint16_t) * 2)) = Endian::host_to_be(value.second);
+    *((uint16_t*)(buffer + sizeof(uint16_t))) = Endian::host_to_be(value[0]);
+    *((uint16_t*)(buffer + sizeof(uint16_t))) = Endian::host_to_be(value[1]);
+    *((uint16_t*)(buffer + sizeof(uint16_t) * 2)) = Endian::host_to_be(value[2]);
     add_option(option(HOME_AGENT_INFO, sizeof(buffer), buffer));
 }
 
@@ -339,10 +357,12 @@ void ICMPv6::target_addr_list(const addr_list_type &value) {
 }
 
 void ICMPv6::add_addr_list(uint8_t type, const addr_list_type &value) {
+    typedef addr_list_type::addresses_type::const_iterator iterator;
+    
     std::vector<uint8_t> buffer;
-    buffer.reserve(value.size() + 6);
-    buffer.insert(buffer.end(), 6, 0);
-    for(addr_list_type::const_iterator it(value.begin()); it != value.end(); ++it)
+    buffer.reserve(value.addresses.size() + 6);
+    buffer.insert(buffer.end(), value.reserved, value.reserved + 6);
+    for(iterator it = value.addresses.begin(); it != value.addresses.end(); ++it)
         buffer.insert(buffer.end(), it->begin(), it->end());
     add_option(option(type, buffer.begin(), buffer.end()));
 }
@@ -361,10 +381,10 @@ void ICMPv6::rsa_signature(const rsa_sign_type &value) {
     add_option(option(RSA_SIGN, buffer.begin(), buffer.end()));
 }
 
-void ICMPv6::timestamp(uint64_t value) {
+void ICMPv6::timestamp(const timestamp_type &value) {
     std::vector<uint8_t> buffer(6 + sizeof(uint64_t));
-    buffer.insert(buffer.begin(), 6, 0);
-    *((uint64_t*)&buffer[6]) = Endian::host_to_be(value);
+    std::copy(value.reserved, value.reserved + 6, buffer.begin());
+    *((uint64_t*)&buffer[6]) = Endian::host_to_be(value.timestamp);
     add_option(option(TIMESTAMP, buffer.begin(), buffer.end()));
 }
 
@@ -394,8 +414,9 @@ void ICMPv6::link_layer_addr(lladdr_type value) {
 
 void ICMPv6::naack(const naack_type &value) {
     uint8_t buffer[6];
-    buffer[0] = value.first;
-    buffer[1] = value.second;
+    buffer[0] = value.code;
+    buffer[1] = value.status;
+    std::copy(value.reserved, value.reserved + 4, buffer + 2);
     add_option(option(NAACK, buffer, buffer + sizeof(buffer)));
 }
 
@@ -536,72 +557,35 @@ void ICMPv6::dns_search_list(const dns_search_list_type &value) {
 // ********************************************************************
 
 ICMPv6::hwaddress_type ICMPv6::source_link_layer_addr() const {
-    const option *opt = search_option(SOURCE_ADDRESS);
-    if(!opt || opt->data_size() != hwaddress_type::address_size)
-        throw option_not_found();
-    return hwaddress_type(opt->data_ptr());
+    return search_and_convert<hwaddress_type>(SOURCE_ADDRESS);
 }
 
 ICMPv6::hwaddress_type ICMPv6::target_link_layer_addr() const {
-    const option *opt = search_option(TARGET_ADDRESS);
-    if(!opt || opt->data_size() != hwaddress_type::address_size)
-        throw option_not_found();
-    return hwaddress_type(opt->data_ptr());
+    return search_and_convert<hwaddress_type>(TARGET_ADDRESS);
 }
 
 ICMPv6::prefix_info_type ICMPv6::prefix_info() const {
-    const option *opt = search_option(PREFIX_INFO);
-    if(!opt || opt->data_size() != 2 + sizeof(uint32_t) * 3 + ipaddress_type::address_size)
-        throw option_not_found();
-    const uint8_t *ptr = opt->data_ptr();
-    prefix_info_type output;
-    output.prefix_len = *ptr++;
-    output.L = (*ptr >> 7) & 0x1;
-    output.A = (*ptr++ >> 6) & 0x1;
-    output.valid_lifetime = Endian::be_to_host(*(uint32_t*)ptr);
-    ptr += sizeof(uint32_t);
-    output.preferred_lifetime = Endian::be_to_host(*(uint32_t*)ptr);
-    output.prefix = ptr + sizeof(uint32_t) * 2;
-    return output;
+    return search_and_convert<prefix_info_type>(PREFIX_INFO);
 }
 
-PDU::serialization_type ICMPv6::redirect_header() const {
-    const option *opt = search_option(REDIRECT_HEADER);
-    if(!opt || opt->data_size() < 6)
-        throw option_not_found();
-    const uint8_t *ptr = opt->data_ptr() + 6;
-    return serialization_type(ptr, ptr + opt->data_size() - 6);
+byte_array ICMPv6::redirect_header() const {
+    return search_and_convert<PDU::serialization_type>(REDIRECT_HEADER);
 }
 
-uint32_t ICMPv6::mtu() const {
-    const option *opt = search_option(MTU);
-    if(!opt || opt->data_size() != sizeof(uint16_t) + sizeof(uint32_t))
-        throw option_not_found();
-    return Endian::be_to_host(*(const uint32_t*)(opt->data_ptr() + sizeof(uint16_t)));
+ICMPv6::mtu_type ICMPv6::mtu() const {
+    return search_and_convert<mtu_type>(MTU);
 }
 
-uint8_t ICMPv6::shortcut_limit() const {
-    const option *opt = search_option(NBMA_SHORT_LIMIT);
-    if(!opt || opt->data_size() != sizeof(uint16_t) + sizeof(uint32_t))
-        throw option_not_found();
-    return *opt->data_ptr();
+ICMPv6::shortcut_limit_type ICMPv6::shortcut_limit() const {
+    return search_and_convert<shortcut_limit_type>(NBMA_SHORT_LIMIT);
 }
 
-uint32_t ICMPv6::new_advert_interval() const {
-    const option *opt = search_option(ADVERT_INTERVAL);
-    if(!opt || opt->data_size() != sizeof(uint16_t) + sizeof(uint32_t))
-        throw option_not_found();
-    return Endian::be_to_host(*(const uint32_t*)(opt->data_ptr() + sizeof(uint16_t)));
+ICMPv6::new_advert_interval_type ICMPv6::new_advert_interval() const {
+    return search_and_convert<new_advert_interval_type>(ADVERT_INTERVAL);
 }
 
 ICMPv6::new_ha_info_type ICMPv6::new_home_agent_info() const {
-    const option *opt = search_option(HOME_AGENT_INFO);
-    if(!opt || opt->data_size() != sizeof(uint16_t) + sizeof(uint32_t))
-        throw option_not_found();
-    return std::make_pair(
-        Endian::be_to_host(*(const uint16_t*)(opt->data_ptr() + sizeof(uint16_t))),
-        Endian::be_to_host(*(const uint16_t*)(opt->data_ptr() + sizeof(uint16_t) * 2))
-    );
+    return search_and_convert<new_ha_info_type>(HOME_AGENT_INFO);
 }
 
 ICMPv6::addr_list_type ICMPv6::source_addr_list() const {
@@ -613,87 +597,150 @@ ICMPv6::addr_list_type ICMPv6::target_addr_list() const {
 }
 
 ICMPv6::addr_list_type ICMPv6::search_addr_list(OptionTypes type) const {
-    const option *opt = search_option(type);
-    if(!opt || opt->data_size() < 6 + ipaddress_type::address_size)
-        throw option_not_found();
+    return search_and_convert<addr_list_type>(type);
+}
+
+ICMPv6::rsa_sign_type ICMPv6::rsa_signature() const {
+    return search_and_convert<rsa_sign_type>(RSA_SIGN);
+}
+
+ICMPv6::timestamp_type ICMPv6::timestamp() const {
+    return search_and_convert<timestamp_type>(TIMESTAMP);
+}
+
+ICMPv6::nonce_type ICMPv6::nonce() const {
+    return search_and_convert<nonce_type>(NONCE);
+}
+
+ICMPv6::ip_prefix_type ICMPv6::ip_prefix() const {
+    return search_and_convert<ip_prefix_type>(IP_PREFIX);
+}   
+
+ICMPv6::lladdr_type ICMPv6::link_layer_addr() const {
+    return search_and_convert<lladdr_type>(LINK_ADDRESS);
+}
+
+ICMPv6::naack_type ICMPv6::naack() const {
+    return search_and_convert<naack_type>(NAACK);
+}
+
+ICMPv6::map_type ICMPv6::map() const {
+    return search_and_convert<map_type>(MAP);
+}
+
+ICMPv6::route_info_type ICMPv6::route_info() const {
+    return search_and_convert<route_info_type>(ROUTE_INFO);
+}
+
+ICMPv6::recursive_dns_type ICMPv6::recursive_dns_servers() const {
+    return search_and_convert<recursive_dns_type>(RECURSIVE_DNS_SERV);
+}
+
+ICMPv6::handover_key_req_type ICMPv6::handover_key_request() const {
+    return search_and_convert<handover_key_req_type>(HANDOVER_KEY_REQ);
+}
+
+ICMPv6::handover_key_reply_type ICMPv6::handover_key_reply() const {
+    return search_and_convert<handover_key_reply_type>(HANDOVER_KEY_REPLY);
+}
+
+ICMPv6::handover_assist_info_type ICMPv6::handover_assist_info() const {
+    return search_and_convert<handover_assist_info_type>(HANDOVER_ASSIST_INFO);
+}
+
+ICMPv6::mobile_node_id_type ICMPv6::mobile_node_identifier() const {
+    return search_and_convert<mobile_node_id_type>(MOBILE_NODE_ID);
+}
+
+ICMPv6::dns_search_list_type ICMPv6::dns_search_list() const {
+    return search_and_convert<dns_search_list_type>(DNS_SEARCH_LIST);
+}
+
+// Options stuff
+
+ICMPv6::addr_list_type ICMPv6::addr_list_type::from_option(const option &opt) 
+{
+    if(opt.data_size() < 6 + ipaddress_type::address_size || (opt.data_size() - 6) % ipaddress_type::address_size != 0)
+        throw malformed_option();
     addr_list_type output;
-    const uint8_t *ptr = opt->data_ptr() + 6, *end = opt->data_ptr() + opt->data_size();
+    const uint8_t *ptr = opt.data_ptr(), *end = opt.data_ptr() + opt.data_size();
+    std::copy(ptr, ptr + 6, output.reserved);
+    ptr += 6;
     while(ptr < end) {
-        if(ptr + ipaddress_type::address_size > end)
-            throw option_not_found();
-        output.push_back(ipaddress_type(ptr));
-        ptr += ipaddress_type::address_size;
+        output.addresses.push_back(ICMPv6::ipaddress_type(ptr));
+        ptr += ICMPv6::ipaddress_type::address_size;
     }
     return output;
 }
 
-ICMPv6::rsa_sign_type ICMPv6::rsa_signature() const {
-    const option *opt = search_option(RSA_SIGN);
-    // 2 bytes reserved + at least 1 byte signature.
-    // 16 == sizeof(rsa_sign_type::key_hash), removed the sizeof
-    // expression since gcc 4.2 doesn't like it
-    if(!opt || opt->data_size() < 2 + 16 + 1)
-        throw option_not_found();
-    const uint8_t *ptr = opt->data_ptr() + 2;
-    rsa_sign_type output;
-    std::copy(ptr, ptr + sizeof(output.key_hash), output.key_hash);
-    ptr += sizeof(output.key_hash);
-    output.signature.assign(ptr, opt->data_ptr() + opt->data_size());
+ICMPv6::naack_type ICMPv6::naack_type::from_option(const option &opt) 
+{
+    if(opt.data_size() != 6)
+        throw malformed_option();
+    return naack_type(*opt.data_ptr(), opt.data_ptr()[1]);
+}
+
+ICMPv6::lladdr_type ICMPv6::lladdr_type::from_option(const option &opt)
+{
+    if(opt.data_size() < 2)
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr();
+    lladdr_type output(*ptr++);
+    output.address.assign(ptr, opt.data_ptr() + opt.data_size());
     return output;
 }
 
-uint64_t ICMPv6::timestamp() const {
-    const option *opt = safe_search_option<std::less>(
-        TIMESTAMP, 6 + sizeof(uint64_t)
-    );
-    return Endian::be_to_host(*(uint64_t*)(opt->data_ptr() + 6));
+ICMPv6::prefix_info_type ICMPv6::prefix_info_type::from_option(const option &opt) 
+{
+    if(opt.data_size() != 2 + sizeof(uint32_t) * 3 + ICMPv6::ipaddress_type::address_size)
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr();
+    prefix_info_type output;
+    output.prefix_len = *ptr++;
+    output.L = (*ptr >> 7) & 0x1;
+    output.A = (*ptr++ >> 6) & 0x1;
+    output.valid_lifetime = Endian::be_to_host(*(uint32_t*)ptr);
+    ptr += sizeof(uint32_t);
+    output.preferred_lifetime = Endian::be_to_host(*(uint32_t*)ptr);
+    output.prefix = ptr + sizeof(uint32_t) * 2;
+    return output;
 }
 
-ICMPv6::nonce_type ICMPv6::nonce() const {
-    const option *opt = safe_search_option<std::equal_to>(
-        NONCE, 0
-    );
-    return nonce_type(opt->data_ptr(), opt->data_ptr() + opt->data_size());
+ICMPv6::rsa_sign_type ICMPv6::rsa_sign_type::from_option(const option &opt)
+{
+    // 2 bytes reserved + at least 1 byte signature.
+    // 16 == sizeof(rsa_sign_type::key_hash), removed the sizeof
+    // expression since gcc 4.2 doesn't like it
+    if(opt.data_size() < 2 + 16 + 1)
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr() + 2;
+    rsa_sign_type output;
+    std::copy(ptr, ptr + sizeof(output.key_hash), output.key_hash);
+    ptr += sizeof(output.key_hash);
+    output.signature.assign(ptr, opt.data_ptr() + opt.data_size());
+    return output;
 }
 
-ICMPv6::ip_prefix_type ICMPv6::ip_prefix() const {
-    const option *opt = safe_search_option<std::less>(
-        IP_PREFIX, 2
-    );
-    const uint8_t *ptr = opt->data_ptr();
+ICMPv6::ip_prefix_type ICMPv6::ip_prefix_type::from_option(const option &opt)
+{
+    // 2 bytes + 4 padding + ipv6 address
+    if(opt.data_size() != 2 + 4 + ICMPv6::ipaddress_type::address_size)
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr();
     ip_prefix_type output;
     output.option_code = *ptr++;
     output.prefix_len = *ptr++;
     // skip padding
     ptr += sizeof(uint32_t);
-    output.address = ipaddress_type(ptr);
-    return output;
-}   
-
-ICMPv6::lladdr_type ICMPv6::link_layer_addr() const {
-    // at least the option_code and 1 byte from the link layer address
-    const option *opt = safe_search_option<std::less>(
-        LINK_ADDRESS, 2
-    );
-    const uint8_t *ptr = opt->data_ptr();
-    lladdr_type output(*ptr++);
-    output.address.assign(ptr, opt->data_ptr() + opt->data_size());
+    output.address = ICMPv6::ipaddress_type(ptr);
     return output;
 }
 
-ICMPv6::naack_type ICMPv6::naack() const {
-    const option *opt = safe_search_option<std::not_equal_to>(
-        NAACK, 6
-    );
-    const uint8_t *ptr = opt->data_ptr();
-    return naack_type(ptr[0], ptr[1]);
-}
-
-ICMPv6::map_type ICMPv6::map() const {
-    const option *opt = safe_search_option<std::not_equal_to>(
-        MAP, 2 + sizeof(uint32_t) + ipaddress_type::address_size
-    );
-    const uint8_t *ptr = opt->data_ptr();
+ICMPv6::map_type ICMPv6::map_type::from_option(const option &opt)
+{
+    if(opt.data_size() != 2 + sizeof(uint32_t) + ipaddress_type::address_size)
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr();
     map_type output;
     output.dist = (*ptr >> 4) & 0x0f;
     output.pref = *ptr++ & 0x0f;
@@ -704,98 +751,98 @@ ICMPv6::map_type ICMPv6::map() const {
     return output;
 }
 
-ICMPv6::route_info_type ICMPv6::route_info() const {
-    const option *opt = safe_search_option<std::less>(
-        ROUTE_INFO, 2 + sizeof(uint32_t)
-    );
-    const uint8_t *ptr = opt->data_ptr();
+ICMPv6::route_info_type ICMPv6::route_info_type::from_option(const option &opt) 
+{
+    if(opt.data_size() < 2 + sizeof(uint32_t))
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr();
     route_info_type output;
     output.prefix_len = *ptr++;
     output.pref = (*ptr++ >> 3) & 0x3;
     output.route_lifetime = Endian::be_to_host(*(uint32_t*)ptr);
     ptr += sizeof(uint32_t);
-    output.prefix.assign(ptr, opt->data_ptr() + opt->data_size());
+    output.prefix.assign(ptr, opt.data_ptr() + opt.data_size());
     return output;
 }
 
-ICMPv6::recursive_dns_type ICMPv6::recursive_dns_servers() const {
-    const option *opt = safe_search_option<std::less>(
-        RECURSIVE_DNS_SERV, 2 + sizeof(uint32_t) + ipaddress_type::address_size
-    );
-    const uint8_t *ptr = opt->data_ptr() + 2, *end = opt->data_ptr() + opt->data_size();
+ICMPv6::recursive_dns_type ICMPv6::recursive_dns_type::from_option(const option &opt)
+{
+    if(opt.data_size() < 2 + sizeof(uint32_t) + ICMPv6::ipaddress_type::address_size)
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr() + 2, *end = opt.data_ptr() + opt.data_size();
     recursive_dns_type output;
     output.lifetime = Endian::be_to_host(*(uint32_t*)ptr);
     ptr += sizeof(uint32_t);
     while(ptr < end) {
-        if(ptr + ipaddress_type::address_size > end)
+        if(ptr + ICMPv6::ipaddress_type::address_size > end)
             throw option_not_found();
         output.servers.push_back(ptr);
-        ptr += ipaddress_type::address_size;
+        ptr += ICMPv6::ipaddress_type::address_size;
     }
     return output;
 }
 
-ICMPv6::handover_key_req_type ICMPv6::handover_key_request() const {
-    const option *opt = safe_search_option<std::less>(
-        HANDOVER_KEY_REQ, 2 + sizeof(uint32_t)
-    );
-    const uint8_t *ptr = opt->data_ptr() + 1, *end = opt->data_ptr() + opt->data_size();
+ICMPv6::handover_key_req_type ICMPv6::handover_key_req_type::from_option(const option &opt)
+{
+    if(opt.data_size() < 2 + sizeof(uint32_t))
+        throw option_not_found();
+    const uint8_t *ptr = opt.data_ptr() + 1, *end = opt.data_ptr() + opt.data_size();
     handover_key_req_type output;
     output.AT = (*ptr++ >> 4) & 0x3;
     // is there enough size for the indicated padding?
-    if(end - ptr < *opt->data_ptr())
-        throw option_not_found();
-    output.key.assign(ptr, ptr + ((end - ptr) - *opt->data_ptr()));
+    if(end - ptr < *opt.data_ptr())
+        throw malformed_option();
+    output.key.assign(ptr, ptr + ((end - ptr) - *opt.data_ptr()));
     return output;
 }
 
-ICMPv6::handover_key_reply_type ICMPv6::handover_key_reply() const {
-    const option *opt = safe_search_option<std::less>(
-        HANDOVER_KEY_REPLY, 2 + sizeof(uint32_t)
-    );
-    const uint8_t *ptr = opt->data_ptr() + 1, *end = opt->data_ptr() + opt->data_size();
+ICMPv6::handover_key_reply_type ICMPv6::handover_key_reply_type::from_option(const option &opt)
+{
+    if(opt.data_size() < 2 + sizeof(uint32_t))
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr() + 1, *end = opt.data_ptr() + opt.data_size();
     handover_key_reply_type output;
     output.AT = (*ptr++ >> 4) & 0x3;
     output.lifetime = Endian::be_to_host(*(uint16_t*)ptr);
     ptr += sizeof(uint16_t);
     // is there enough size for the indicated padding?
-    if(end - ptr < *opt->data_ptr())
-        throw option_not_found();
-    output.key.assign(ptr, ptr + ((end - ptr) - *opt->data_ptr()));
+    if(end - ptr < *opt.data_ptr())
+        throw malformed_option();
+    output.key.assign(ptr, ptr + ((end - ptr) - *opt.data_ptr()));
     return output;
 }
 
-ICMPv6::handover_assist_info_type ICMPv6::handover_assist_info() const {
-    const option *opt = safe_search_option<std::less>(
-        HANDOVER_ASSIST_INFO, 2
-    );
-    const uint8_t *ptr = opt->data_ptr(), *end = ptr + opt->data_size();
+ICMPv6::handover_assist_info_type ICMPv6::handover_assist_info_type::from_option(const option &opt)
+{
+    if(opt.data_size() < 2)
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr(), *end = ptr + opt.data_size();
     handover_assist_info_type output;
     output.option_code = *ptr++;
     if((end - ptr - 1) < *ptr)
-        throw option_not_found();
+        throw malformed_option();
     output.hai.assign(ptr + 1, ptr + 1 + *ptr);
     return output;
 }
 
-ICMPv6::mobile_node_id_type ICMPv6::mobile_node_identifier() const {
-    const option *opt = safe_search_option<std::less>(
-        MOBILE_NODE_ID, 2
-    );
-    const uint8_t *ptr = opt->data_ptr(), *end = ptr + opt->data_size();
+ICMPv6::mobile_node_id_type ICMPv6::mobile_node_id_type::from_option(const option &opt)
+{
+    if(opt.data_size() < 2)
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr(), *end = ptr + opt.data_size();
     mobile_node_id_type output;
     output.option_code = *ptr++;
     if((end - ptr - 1) < *ptr)
-        throw option_not_found();
+        throw malformed_option();
     output.mn.assign(ptr + 1, ptr + 1 + *ptr);
     return output;
 }
 
-ICMPv6::dns_search_list_type ICMPv6::dns_search_list() const {
-    const option *opt = safe_search_option<std::less>(
-        DNS_SEARCH_LIST, 2 + sizeof(uint32_t)
-    );
-    const uint8_t *ptr = opt->data_ptr(), *end = ptr + opt->data_size();
+ICMPv6::dns_search_list_type ICMPv6::dns_search_list_type::from_option(const option &opt)
+{
+    if(opt.data_size() < 2 + sizeof(uint32_t))
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr(), *end = ptr + opt.data_size();
     dns_search_list_type output;
     output.lifetime = Endian::be_to_host(*(uint32_t*)(ptr + 2));
     ptr += 2 + sizeof(uint32_t);
@@ -815,5 +862,35 @@ ICMPv6::dns_search_list_type ICMPv6::dns_search_list() const {
     }
     return output;
 }
+
+ICMPv6::timestamp_type ICMPv6::timestamp_type::from_option(const option &opt)
+{
+    if(opt.data_size() != 6 + sizeof(uint64_t))
+        throw malformed_option();
+    timestamp_type output(Endian::be_to_host(*(uint64_t*)(opt.data_ptr() + 6)));
+    std::copy(opt.data_ptr(), opt.data_ptr() + 6, output.reserved);
+    return output;
+}
+
+ICMPv6::shortcut_limit_type ICMPv6::shortcut_limit_type::from_option(const option &opt)
+{
+    if(opt.data_size() != 6)
+        throw malformed_option();
+    const uint8_t *ptr = opt.data_ptr();
+    shortcut_limit_type output(*ptr++);
+    output.reserved1 = *ptr++;
+    output.reserved2 = Endian::be_to_host(*(uint32_t*)ptr);
+    return output;
+}
+
+ICMPv6::new_advert_interval_type ICMPv6::new_advert_interval_type::from_option(const option &opt)
+{
+    if(opt.data_size() != 6)
+        throw malformed_option();
+    new_advert_interval_type output;
+    output.reserved = Endian::be_to_host(*(uint16_t*)opt.data_ptr());
+    output.interval = Endian::be_to_host(*(uint32_t*)(opt.data_ptr() + sizeof(uint16_t)));
+    return output;
+}
 }
 
